#include <stdio.h>
#include <math.h>
#include "defs.h"
#include "par_temp.h"

PAR_SYSTEMS * clone_par_systems(PAR_SYSTEMS * ps1)
{
   PAR_SYSTEMS * ps2;
   calloc_(ps2,1);

   ps2[0] = ps1[0];

   calloc_(ps2->min_p,1);
   ps2->min_p[0] = ps1->min_p[0];
   calloc_(ps2->min_p->var,ps1->min_p->num_params);

   ps2->var = ps2->min_p->var;

   memcpy_(ps2->var,ps1->var,ps2->monte->num_var);
   return ps2;
}

PAR_TEMP * new_par_temp(int num_systems, PAR_SYSTEMS * par_sys, float T_low, float T_high, long idum)
{
   int i;
   PAR_TEMP * par_temp;

   calloc_(par_temp,1);
   par_temp->num_systems = num_systems;

   calloc_(par_temp->sys, num_systems);
   calloc_(par_temp->beta, num_systems);
   calloc_(par_temp->num_attempts, num_systems);
   calloc_(par_temp->num_accepted, num_systems);

   par_temp->idum = idum;
   if (num_systems == 1)
   {
      par_temp->beta[0] = 1/T_low;
   }
   else
   {
      for(i=0; i < num_systems; i++)
      {
	 par_temp->beta[i] = 1.0/(T_low + (T_high-T_low)*pow(((float) i)/(num_systems-1),6));
	 par_temp->sys[i] = clone_par_systems(par_sys);
	 par_temp->sys[i]->monte = clone_monte(par_sys->monte);
	 par_temp->sys[i]->monte->beta = par_temp->beta[i];
	 par_temp->sys[i]->monte->idum = idum-1-i;
      }
   }

   return par_temp;
}

float move_par_temp(PAR_TEMP * par_temp, int num_moves)
{
float move_one_sweep(DATA * data, MIN_PARAMETERS * min_p,  MONTE * monte, float * var, int num_its_in);
   void print_params_(MIN_PARAMETERS * min_p, float * p_best, FILE * file);
   float ran1(long *idum);
   int i;
   int sys_num;
   float * e_min;
   PAR_SYSTEMS ** sys = par_temp->sys;
   alloca_0(e_min,par_temp->num_systems);

   for(i=0; i < num_moves; i++)
   {
      for(sys_num=0; sys_num < par_temp->num_systems; sys_num++)
      {
	 PAR_SYSTEMS * syst = sys[sys_num];
	 e_min[sys_num] = move_one_sweep(syst->data, syst->min_p, syst->monte, syst->var, 8 * syst->monte->num_var);
      }
      for(sys_num=0; sys_num < (par_temp->num_systems-1); sys_num++)
      {
	 float beta_1 = par_temp->beta[sys_num];
	 float beta_2 = par_temp->beta[sys_num+1];
	 float e_1 = par_temp->sys[sys_num]->monte->energy;
	 float e_2 = par_temp->sys[sys_num+1]->monte->energy;

	 if (( ran1(&(par_temp->idum)) < exp((beta_2-beta_1)*(e_2-e_1))))
	 {
	    SWAP(par_temp->sys[sys_num],par_temp->sys[sys_num+1]);
	    SWAP(par_temp->sys[sys_num]->monte->beta,par_temp->sys[sys_num+1]->monte->beta);
	    par_temp->num_accepted[sys_num]++;
	 }
	 par_temp->num_attempts[sys_num]++;
      }
      if (i%10 == 0)
      {
	 printf("iteration %d\n",i);
	 printf("\n");
	 for(sys_num=0; sys_num < (par_temp->num_systems-1); sys_num++)
	 {
	    printf("sys_num = %d, T = %f, acceptance_rate = %f, num_attempts = %d, energy = %f, e_min = %f \n", 
		  sys_num, 1.0/par_temp->beta[sys_num], ((float) par_temp->num_accepted[sys_num])/par_temp->num_attempts[sys_num], 
		  par_temp->num_attempts[sys_num], par_temp->sys[sys_num]->monte->energy, e_min[sys_num]); 
	 }
      }

      if (i%50 == 0)
      {
	 memset_(par_temp->num_attempts,0, par_temp->num_systems);
	 memset_(par_temp->num_accepted,0, par_temp->num_systems);
      }
      if (i%10 == 0)
	 print_params_(par_temp->sys[0]->min_p, par_temp->sys[0]->min_p->var,stdout); 
   }
   return e_min[0];
}

