#include "ParTemp.h"

#define float double
ParTemp::ParTemp(ParTempable * sys, int NumSystems, float BetaMin, float BetaMax, int PrintInterval, int StepsPerMove, int NumMoves, RandNumGenerator * Rand) {
   this->NumSystems = NumSystems;
   this->BetaMin = BetaMin;
   this->BetaMax = BetaMax;
   this->PrintInterval = PrintInterval;
   this->NumMoves = NumMoves;
   this->Rand = Rand;
   this->StepsPerMove = StepsPerMove;

   initBeta();
   Systems = new (ParTempable *)[NumSystems];
   for(int i=0; i < NumSystems; i++) 
      Systems[i] = sys->clone();
}

ParTemp::ParTemp(ParTempable * sys, int NumSystems, float BetaMin, float BetaMax, int PrintInterval, int StepsPerMove, int NumMoves, long RanSeed) {

   Rand = new RandNumGenerator(RanSeed);

   ParTemp(sys, NumSystems, BetaMin, BetaMax, PrintInterval, StepsPerMove, NumMoves, Rand);
}


void ParTemp::initBeta() {
   Beta = new float[NumSystems];
   float norm = (1/BetaMin-1/BetaMax)/(NumSystems*NumSystems);
   for(int i=0; i < NumSystems; i++) {
      Beta[i] = 1.0/(norm * i*i + 1/BetaMax);
   }
}

ParTempable * ParTemp::getParTempable(int num) {
   return Systems[num];
}
void ParTemp::setMonitor(int MaxSize) {
   monitor = new Monitor<int>[NumSystems](MaxSize);
}


float ParTemp::move() {
   float Emin[NumSystems];
   for (int i=0; i < NumMoves; i++) {
      for(int SysNum=0; SysNum < NumSystems; SysNum++) {
	 ParTempable * System = Systems[SysNum];
	 System->moveParTemp(StepsPerMove);
	 Emin[SysNum] = System->getMinEnergy();
      }
      for(int SysNum=0; SysNum < NumSystems-1; SysNum++) {
	 float Beta1 = Beta[SysNum];
	 float Beta2 = Beta[SysNum+1];
	 float Energy1 = Systems[SysNum]->getEnergy();
	 float Energy2 = Systems[SysNum+1]->getEnergy();

	 if (Rand->nextFloat() < exp((Beta2-Beta1)*(Energy2-Energy1))) {
	    SWAP(Systems[SysNum], Systems[SysNum+1]);
	    Systems[SysNum]->setBeta(Beta1);
	    Systems[SysNum+1]->setBeta(Beta2);
	    monitor[SysNum].add(1);
	 }
	 else {
	    monitor[SysNum].add(0);
	 }
      }

      if (PrintInterval > 0)
	 if (i % PrintInterval == 0) {
	    cout <<  "iteration " << i << ":" << endl;
	    for(int SysNum=0; SysNum < NumSystems; SysNum++) {
	       cout << "Sys Num " << SysNum << " T= " << 1/Beta[SysNum] << "accept rate = " 
		  << ((float) monitor[SysNum].getSum())/monitor[SysNum].getSize()
		  << "Num Attempts = " << monitor[SysNum].getSum() << "energy= " 
		  << Systems[SysNum]->getEnergy() <<  "min e= " << Systems[SysNum]->getMinEnergy() << endl;
	    }
	 }
   }
}
#undef float
